#ifndef XUNIX_H
#define XUNIX_H

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <csignal>
#include <stdexcept>
#include <string>

namespace xunix {

  // xunix::exception converts unix errors associated with errno into
  // C++ exceptions.
  class exception : public std::exception{
  public:
    exception(const std::string& prefix);
    virtual ~exception() throw();
    virtual const char* what() const throw();
    virtual int error_number() const throw();
  private:
    void init(const std::string& prefix);
    std::string _what;
    // "errno" is sometimes a macro.  Use something else.
    int _error_number;
  };

  //
  // File Operations
  //

  int open(const char* filename, int flags, mode_t mode = 0);
  void close(int fd);
  void fcntl(int fd, int cmd, struct flock* lock_flock);

  //
  // Signals
  //

  void sigaction(int signum,
                 const struct sigaction* act,
                 struct sigaction* oldact) throw(xunix::exception);

  void sigprocmask(int how, const sigset_t* sig_set, sigset_t* old_sig_set);
  void sigsuspend(const sigset_t* mask);
   int sigwaitinfo(const sigset_t* sig_set, siginfo_t* sig_info);

  void sigemptyset(sigset_t* sigset);
  void sigfillset(sigset_t* sigset);
  void sigaddset(sigset_t* sigset, int signum);
  void sigdelset(sigset_t* sigset, int signum);
   int sigismember(const sigset_t* sigset, int signum);

   void pause(void);

}

#endif  /*  XUNIX_H  */
